--[[
Improves the exhaust smoke effect by making it darker and more realistic. Mod inspired by darker exhaust smoke mod by Rival.

Author:     w33zl
Version:    1.0.0
Modified:   2025-02-23

Changelog:

]]

local NO_DEF_COLOR = {0.10, 0.10, 0.10, 0.4 }
local NO_DEF_MAX_FACTOR = 0.7
local LOW_TEMP_FACTOR = 1.5

local LOW_TEMP_COLOR = {1, 0.015, 0.02, 0.4 }
local LOW_TEMP_NO_DEF_COLOR = {0.12, 0.12, 0.08, 0.4 }
local LOW_TEMP_FACTOR_MAX = 2.5 --1.5
local LOW_TEMP_FACTOR_MIN = 1.05

local function cloneColorDarker(color)
	return {color[1]*0.4, color[2]*0.4, color[3]*0.3, color[4]}
end

Motorized.onExhaustEffectI3DLoaded = Utils.appendedFunction(Motorized.onExhaustEffectI3DLoaded, function(self, ...)
	local consumesAdBlue = false
	local spec = self.spec_motorized
	local consumers = spec.consumersByFillTypeName

	if consumers ~= nil then
		for _, consumerType in pairs(consumers) do
			if consumerType.fillType == FillType.DEF then
				consumesAdBlue = true
				break
			end
		end
	end

	for _, exhaustEffect in ipairs(spec.exhaustEffects) do
		-- printf("Default minRpmColor: %f %f %f %f", effect.minRpmColor[1], effect.minRpmColor[2], effect.minRpmColor[3], effect.minRpmColor[4])
		local minFactor = exhaustEffect.minRpmColor[4]
		local maxFactor = exhaustEffect.maxRpmColor[4]

		if not consumesAdBlue then
			exhaustEffect.lowTempMinRpmColor = table.clone(LOW_TEMP_NO_DEF_COLOR)
			exhaustEffect.lowTempMaxRpmColor = table.clone(LOW_TEMP_NO_DEF_COLOR)

			--HACK: 
			-- effect.lowTempMinRpmColor[3] = 1
			-- effect.lowTempMaxRpmColor[3] = 1
	
			exhaustEffect.minRpmColor = table.clone(NO_DEF_COLOR)
			exhaustEffect.maxRpmColor = table.clone(NO_DEF_COLOR)
			minFactor = math.min(minFactor * 5, 5)
			maxFactor = math.min(maxFactor * 10, 10)
		else
			-- effect.lowTempMinRpmColor = table.clone(LOW_TEMP_COLOR)
			-- effect.lowTempMaxRpmColor = table.clone(LOW_TEMP_COLOR)
			exhaustEffect.lowTempMinRpmColor = cloneColorDarker(exhaustEffect.minRpmColor)
			exhaustEffect.lowTempMaxRpmColor = cloneColorDarker(exhaustEffect.maxRpmColor)

			--HACK: 
			-- effect.lowTempMinRpmColor[1] = 1
			-- effect.lowTempMaxRpmColor[1] = 1
	
			maxFactor = maxFactor * 1.5
		end

		exhaustEffect.minRpmColor[4] = minFactor
		exhaustEffect.maxRpmColor[4] = maxFactor

		exhaustEffect.defaultMinRpmColor = exhaustEffect.minRpmColor
		exhaustEffect.defaultMaxRpmColor = exhaustEffect.maxRpmColor
	end
end)

Motorized.updateMotorTemperature = Utils.appendedFunction(Motorized.updateMotorTemperature, function(self, ...)
	-- if not self:getIsMotorStarted() then
	-- 	return
	-- end

	local spec = self.spec_motorized

	-- printf("temp: %f min: %f max: %f, fanEnable: %f, fanDisable: %f", spec.motorTemperature.value, spec.motorTemperature.valueMin, spec.motorTemperature.valueMax, spec.motorFan.enableTemperature, spec.motorFan.disableTemperature)
	for _, exhaustEffect in ipairs(spec.exhaustEffects) do
		if spec.motorTemperature.value < spec.motorFan.disableTemperature then
			exhaustEffect.minRpmColor = exhaustEffect.lowTempMinRpmColor
			exhaustEffect.maxRpmColor = exhaustEffect.lowTempMaxRpmColor

			local minTemp = 0 --spec.motorTemperature.valueMin --NOTE: any benefit of using minValue?
			local maxTemp = spec.motorTemperature.valueMax -- TODO: We do not need this?
			local temp = math.max(spec.motorTemperature.value, minTemp)
			local exhaustStrengthFactor = (temp-minTemp)/(spec.motorFan.disableTemperature-minTemp) * (LOW_TEMP_FACTOR_MIN-LOW_TEMP_FACTOR_MAX) + LOW_TEMP_FACTOR_MAX
			-- local tempFactor = math.clamp((temp - spec.motorTemperature.valueMin) / (spec.motorTemperature.valueMax - spec.motorTemperature.valueMin), 0, 1)
	
			local minFactor = exhaustEffect.defaultMinRpmColor[4]
			local maxFactor = exhaustEffect.defaultMaxRpmColor[4]
			
			exhaustEffect.minRpmColor[4] = minFactor * exhaustStrengthFactor
			exhaustEffect.maxRpmColor[4] = maxFactor * exhaustStrengthFactor

			Log:debug("temp: %f xFactor: %f", spec.motorTemperature.value, exhaustStrengthFactor)
		else
			exhaustEffect.minRpmColor = exhaustEffect.defaultMinRpmColor
			exhaustEffect.maxRpmColor = exhaustEffect.defaultMaxRpmColor

		end

		-- Boost the effect when starting the engine
		if self:getMotorState() == MotorState.STARTING then
			-- effect.minRpmColor[1] = 1
			-- effect.maxRpmColor[1] = 1

			local x = math.random(80, 150) / 100
			exhaustEffect.minRpmColor[4] = exhaustEffect.minRpmColor[4] * x
			exhaustEffect.maxRpmColor[4] = exhaustEffect.maxRpmColor[4] * x
		-- else
		-- 	effect.minRpmColor[1] = effect.defaultMinRpmColor[1]
		-- 	effect.maxRpmColor[1] = effect.defaultMaxRpmColor[1]
		end
	end
end)

